@@ -17,7 +17,7 @@ from threading import Timer
 
 from firex_flame.main_app import start_flame
 from firex_flame.flame_helper import get_flame_debug_dir, get_flame_pid_file_path, DEFAULT_FLAME_TIMEOUT, \
-    BrokerConsumerConfig, get_flame_url, wait_until, FlameServerConfig
+    BrokerConsumerConfig, get_flame_url, wait_until, FlameServerConfig, REVOKE_REASON_KEY, REVOKE_TIMESTAMP_KEY
 # Prevent dependencies from taking module loading hit of pkg_resources.
 from firexapp.submit.submit import OptionalBoolean
 
@@ -128,6 +128,7 @@ def _config_logging(root_logs_dir):
 
 
 def _create_run_metadata(cli_args):
+    #FIXME: should be a dataclass.
     return {
         'uid': cli_args.uid,
         'logs_dir': cli_args.logs_dir,
@@ -140,6 +141,8 @@ def _create_run_metadata(cli_args):
         'flame_url': None,
         'firex_bin': cli_args.firex_bin_path,
         'root_uuid': None,
+        REVOKE_REASON_KEY: None,
+        REVOKE_TIMESTAMP_KEY: None,
     }
 
 
