@@ -3,6 +3,7 @@ Aggregates events in to the task data model.
 """
 from datetime import datetime
 import logging
+from typing import Any, Optional
 
 from firexkit.task import FIREX_REVOKE_COMPLETE_EVENT_TYPE
 from firexapp.events.model import ADDITIONAL_CHILDREN_KEY, EXTERNAL_COMMANDS_KEY
@@ -63,7 +64,7 @@ FIELD_CONFIG = {
     'chain_depth': {'copy_celery': True, 'slim_field': True},
     'firex_result': {'copy_celery': True},
     'traceback': {'copy_celery': True},
-    'exception': {'copy_celery': True, 'slim_field': True},
+    'exception': {'copy_celery': True},
     'long_name': {
         'copy_celery': True,
         'transform_celery': lambda e: {'name': e['long_name'].split('.')[-1]},
@@ -84,7 +85,10 @@ FIELD_CONFIG = {
     'called_as_orig': {
         'copy_celery': True,
     },
-    'first_started': {'slim_field': True, 'aggregate_keep_initial': True},
+    'first_started': {
+        'slim_field': True,
+        'aggregate_keep_initial': True,
+    },
     'url': {
         # TODO: only for backwards compat. Can use log_filepath.
         'transform_celery': lambda e: {'logs_url': e['url']},
@@ -93,11 +97,17 @@ FIELD_CONFIG = {
         'transform_celery': lambda e: {'logs_url': e['log_filepath']},
     },
     'local_received': {
-        # Note first_started is never overwritten by aggregation.
-        'transform_celery': lambda e: {'first_started': e['local_received']},
+        'transform_celery': lambda e: {
+            # Note first_started is never overwritten by aggregation.
+            'first_started': e['local_received'],
+            'latest_timestamp': e['local_received'],
+        },
     },
     'states': {'aggregate_merge': True},
-    'exception_cause_uuid': {'copy_celery': True},
+    'exception_cause_uuid': {
+        'copy_celery': True,
+        'slim_field': True,
+    },
     EXTERNAL_COMMANDS_KEY: {
         'copy_celery': True,
         'aggregate_merge': True,
@@ -113,6 +123,12 @@ FIELD_CONFIG = {
     },
     'cached_result_from': {
         'copy_celery': True,
+    },
+    'latest_timestamp': {
+        'slim_field': True,
+    },
+    'pid': {
+        'copy_celery': True,
     }
 }
 
@@ -202,9 +218,9 @@ class FlameEventAggregator:
     """ Aggregates many events in to the task data model. """
 
     def __init__(self):
-        self.tasks_by_uuid = {}
-        self.new_task_num = 1
-        self.root_uuid = None
+        self.tasks_by_uuid : dict[str, dict[str, Any]]= {}
+        self.new_task_num : int = 1
+        self.root_uuid : Optional[str] = None
 
     def aggregate_events(self, events):
         new_data_by_task_uuid = {}
@@ -219,7 +235,7 @@ class FlameEventAggregator:
     def generate_incomplete_events(self):
         """
         Unfortunately, if a run terminates ungracefully, incomplete tasks will never arrive at a
-        terminal runstate. The 'task-incomplete' runstate is a fake (non-backend) terminal runstate
+        terminal runstate. The 'task-incomplete' runstate is a fake (non-Celery) terminal runstate
         that is generated here so that the UI can show a non-incomplete runstate.
         :return:
         """
@@ -283,3 +299,14 @@ class FlameEventAggregator:
             changes_by_task_uuid[task_uuid] = dict(task) if is_new_task else dict(changed_data)
 
         return changes_by_task_uuid
+
+    def all_tasks_complete(self) -> bool:
+        if not self.tasks_by_uuid:
+            # do not want "no tasks received yet"
+            # to look like "all tasks complete"
+            return False
+
+        return all(
+            t.get('state') in COMPLETE_STATES
+            for t in self.tasks_by_uuid.values()
+        )
